@@ -5,9 +5,8 @@
  * Author: Chris Lahey <clahey@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -123,7 +122,7 @@ e_contact_editor_address_destroy (GtkObject *object)
 
 	if (e_contact_editor_address->gui)
 		gtk_object_unref(GTK_OBJECT(e_contact_editor_address->gui));
-	e_card_delivery_address_free(e_contact_editor_address->address);
+	e_card_delivery_address_unref(e_contact_editor_address->address);
 }
 
 GtkWidget*
@@ -145,8 +144,7 @@ e_contact_editor_address_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	
 	switch (arg_id){
 	case ARG_ADDRESS:
-		if (e_contact_editor_address->address)
-			e_card_delivery_address_free(e_contact_editor_address->address);
+		e_card_delivery_address_unref(e_contact_editor_address->address);
 		e_contact_editor_address->address = e_card_delivery_address_copy(GTK_VALUE_POINTER (*arg));
 		fill_in_info(e_contact_editor_address);
 		break;
@@ -190,7 +188,7 @@ e_contact_editor_address_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	switch (arg_id) {
 	case ARG_ADDRESS:
 		extract_info(e_contact_editor_address);
-		GTK_VALUE_POINTER (*arg) = e_card_delivery_address_copy(e_contact_editor_address->address);
+		GTK_VALUE_POINTER (*arg) = e_card_delivery_address_ref(e_contact_editor_address->address);
 		break;
 	case ARG_IS_READ_ONLY:
 		GTK_VALUE_BOOL (*arg) = e_contact_editor_address->editable ? TRUE : FALSE;
