@@ -4,9 +4,8 @@
  * Copyright (C) 2001  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -222,18 +221,16 @@ checkbox_toggle_cb (GtkToggleButton *tb,
 static BonoboControl *
 create_checkboxes_control (GnomeCardImporter *importer)
 {
-	GtkWidget *label, *hbox;
+	GtkWidget *hbox;
 	BonoboControl *control;
 
 	hbox = gtk_hbox_new (FALSE, 2);
 
-	label = gtk_label_new (_("GnomeCard:"));
 	importer->addresses = gtk_check_button_new_with_label (_("Addressbook"));
 	gtk_signal_connect (GTK_OBJECT (importer->addresses), "toggled",
 			    GTK_SIGNAL_FUNC (checkbox_toggle_cb),
 			    &importer->do_addresses);
 
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), importer->addresses, FALSE, FALSE, 0);
 
 	gtk_widget_show_all (hbox);
