@@ -7,9 +7,8 @@
  * Copyright 2001 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public 
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,6 +31,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <ctype.h>
 
 #include <glib.h>
 
@@ -76,16 +76,40 @@ typedef struct {
 	EBook *book;
 
 	Bonobo_ConfigDatabase db;
+
+	/* GUI */
+	GtkWidget *dialog;
+	GtkWidget *label;
+	GtkWidget *progressbar;
 } PineImporter;
 
 typedef struct {
 	char *parent;
 	char *foldername;
 	char *path;
+	gboolean folder;
 } PineFolder;
 
 static void import_next (PineImporter *importer);
 
+static GtkWidget *
+create_importer_gui (PineImporter *importer)
+{
+	GtkWidget *dialog;
+
+	dialog = gnome_message_box_new (_("Evolution is importing your old Pine data"), GNOME_MESSAGE_BOX_INFO, NULL);
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Importing..."));
+
+	importer->label = gtk_label_new (_("Please wait"));
+	importer->progressbar = gtk_progress_bar_new ();
+	gtk_progress_set_activity_mode (GTK_PROGRESS (importer->progressbar), TRUE);
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
+			    importer->label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
+			    importer->progressbar, FALSE, FALSE, 0);
+	return dialog;
+}
+
 static void
 pine_store_settings (PineImporter *importer)
 {
@@ -106,65 +130,6 @@ pine_restore_settings (PineImporter *importer)
 
 }
 
-/* Pass in handle so we can get the next line if we need to */
-static char *
-get_field (char **start,
-	   FILE *handle)
-{
-	char line[4096];
-	char *end, *field;
-	int length;
-
-	/* if this was the last line, just return */
-	if (*start == NULL) {
-		return NULL;
-	}
-
-	/* if start is just \n then we need the next line */
-	if (**start == '\n') {
-		d(g_warning ("Need next line"));
-		if (fgets (line, 4096, handle) == NULL) {
-			g_warning ("Hmmm, no next line");
-			return NULL;
-		}
-
-		if (line[0] != ' ' || line[1] != ' ' || line[2] != ' ') {
-			g_warning ("Next line was not a continuation line\n"
-				   "Oppps: %s", line);
-			return NULL;
-		}
-
-		*start = line + 3;
-	}
-
-	if (**start == '\t') {
-		/* Blank field */
-
-		*start += 1;
-		return NULL;
-	}
-
-	end = strchr (*start, '\t');
-	if (end == NULL) {
-		/* the line was the last comment, so just return the line */
-		length = strlen (line);
-		line[length - 1] = 0;
-
-		field = g_strdup (*start);
-		
-		*start = NULL;
-	} else {
-		/* Null the end */
-		*end = 0;
-
-		field = g_strdup (*start);
-		*start = end + 1;
-	}
-
-	d(g_warning ("Found %s", field));
-	return field;
-}
-
 /* A very basic address spliter.
    Returns the first email address
    denoted by <address> */
@@ -173,6 +138,10 @@ parse_address (const char *address)
 {
 	char *addr_dup, *result, *start, *end;
 
+	if (address == NULL) {
+		return NULL;
+	}
+
 	addr_dup = g_strdup (address);
 	start = strchr (addr_dup, '<');
 	if (start == NULL) {
@@ -196,6 +165,41 @@ parse_address (const char *address)
 	return result;
 }
 
+static void
+add_card_cb (EBook *book, 
+	     EBookStatus status,
+	     const char *id,
+	     gpointer closure)
+{
+	gtk_object_unref (GTK_OBJECT (closure));
+}
+
+static void
+parse_line (EBook *book,
+	    char *line)
+{
+	char **strings;
+	ECardName *name;
+	ECard *card;
+	EList *list;
+
+	card = e_card_new ("");
+	strings = g_strsplit (line, "\t", 3);
+	if (strings[0] && strings[1] && strings[2]) {
+		name = e_card_name_from_string (strings[1]);
+		gtk_object_set (GTK_OBJECT (card),
+				"nickname", strings[0],
+				"full_name", strings[1],
+				"name", name, NULL);
+		gtk_object_get (GTK_OBJECT (card),
+				"email", &list,
+				NULL);
+		e_list_append (list, strings[2]);
+		g_strfreev (strings);
+		e_book_add_card (book, card, add_card_cb, card);
+	}
+}
+
 static void
 import_addressfile (EBook *book,
 		    EBookStatus status,
@@ -203,8 +207,9 @@ import_addressfile (EBook *book,
 {
 	char *addressbook;
 	FILE *handle;
-	char line[4096];
-
+	char line[2 * 1024];
+	int which = 0;
+	char *lastline = NULL;
 	PineImporter *importer = user_data;
 
 	addressbook = gnome_util_prepend_user_home (".addressbook");
@@ -216,91 +221,40 @@ import_addressfile (EBook *book,
 		return;
 	}
 
-	while (fgets (line, 4096, handle) != NULL) {
-		char *nick, *fullname, *address, *comment, *fcc, *email = NULL;
-		char *start;
-		gboolean distrib = FALSE;
-
-		start = line;
-		nick = get_field (&start, handle);
-		d(g_print ("Nick: %s\n", nick));
-
-		fullname = get_field (&start, handle);
-		d(g_print ("Fullname: %s\n", fullname));
-
-		address = get_field (&start, handle);
-		d(g_print ("Address: %s\n", address));
-
-		if (*address == '(') {
-			char nextline[4096];
-			/* Fun, it's a distribution list */
-			distrib = TRUE;
-			
-			/* if the last char of address == ) then this is the
-			   full list */
-			while (fgets (nextline, 4096, handle)) {
-				char *bracket;
-				if (nextline[0] != ' ' ||
-				    nextline[1] != ' ' ||
-				    nextline[2] != ' ') {
-					/* Not a continuation line */
-					start = nextline;
-					break;
-				}
-
-				bracket = strchr (nextline, ')');
-				if (bracket != NULL &&
-				    *(bracket + 1) == '\t') {
-
-					*(bracket + 1) = 0;
-					d(g_print ("More addresses: %s\n", nextline));
-					start = bracket + 2;
-					break;
-				} else {
-					d(g_print ("More addresses: %s\n", nextline));
-				}
-			}
-		} else {
-			email = parse_address (address);
-		}
-
-		fcc = get_field (&start, handle);
-		d(g_print ("FCC: %s\n", fcc));
-
-		comment = get_field (&start, handle);
-		d(g_print ("Comment: %s\n", comment));
-
-		if (distrib == FALSE) {
-			/* This was not a distribution list...
-			   Evolution doesn't handle that yet (070501)
-			   Fixme when it does */
-			ECard *card = e_card_new ("");
-			ECardSimple *simple = e_card_simple_new (card);
-
-			if (fullname != NULL)
-				e_card_simple_set (simple, E_CARD_SIMPLE_FIELD_FULL_NAME,
-						   fullname);
-			else
-				e_card_simple_set (simple, E_CARD_SIMPLE_FIELD_FILE_AS,
-						   nick);
-
-			e_card_simple_set (simple, E_CARD_SIMPLE_FIELD_EMAIL,
-					   email);
-			e_card_simple_set (simple, E_CARD_SIMPLE_FIELD_NOTE,
-					   comment);
-			e_card_simple_set (simple, E_CARD_SIMPLE_FIELD_NICKNAME,
-					   nick);
-			e_book_add_card (importer->book, simple->card, NULL, NULL);
-		}
+	while (fgets (line + which * 1024, 1024, handle)) {
+		int length;
+		char *thisline = line + which * 1024;
 		
-		g_free (nick);
-		g_free (email);
-		g_free (address);
-		g_free (comment);
-		g_free (fcc);
-		g_free (fullname);
+		length = strlen (thisline);
+		if (thisline[length - 1] == '\n') {
+			line[--length] = 0;
+		}
+
+		if (lastline && *thisline && isspace ((int) *thisline)) {
+			char *temp;
+
+			while (*thisline && isspace ((int) *thisline)) {
+				thisline++;
+			}
+			temp = lastline;
+			lastline = g_strdup_printf ("%s%s", lastline, thisline);
+			g_free (temp);
+			continue;
+		}
+
+		if (lastline) {
+			parse_line (book, lastline);
+			g_free (lastline);
+		}
+
+		lastline = g_strdup (thisline);
 	}
-	
+
+	if (lastline) {
+		parse_line (book, lastline);
+		g_free (lastline);
+	}
+
 	fclose (handle);
 	if (importer->do_mail == FALSE) {
 		if (importer->db != CORBA_OBJECT_NIL) {
@@ -332,6 +286,21 @@ import_addressbook (PineImporter *importer)
 	g_free (uri);
 }
 
+static gboolean
+importer_timeout_fn (gpointer data)
+{
+	PineImporter *importer = (PineImporter *) data;
+	CORBA_Object objref;
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+	objref = bonobo_object_corba_objref (BONOBO_OBJECT (importer->listener));
+	GNOME_Evolution_Importer_processItem (importer->importer, objref, &ev);
+	CORBA_exception_free (&ev);
+
+	return FALSE;
+}
+
 static void
 importer_cb (EvolutionImporterListener *listener,
 	     EvolutionImporterResult result,
@@ -342,7 +311,23 @@ importer_cb (EvolutionImporterListener *listener,
 	CORBA_Object objref;
 	CORBA_Environment ev;
 
+	if (result == EVOLUTION_IMPORTER_NOT_READY ||
+	    result == EVOLUTION_IMPORTER_BUSY) {
+		gtk_timeout_add (1000, importer_timeout_fn, data);
+		return;
+	}
+
 	if (more_items) {
+		GtkAdjustment *adj;
+		float newval;
+
+		adj = GTK_PROGRESS (importer->progressbar)->adjustment;
+		newval = adj->value + 1;
+		if (newval > adj->upper) {
+			newval = adj->lower;
+		}
+
+		gtk_progress_set_value (GTK_PROGRESS (importer->progressbar), newval);
 		CORBA_exception_init (&ev);
 		objref = bonobo_object_corba_objref (BONOBO_OBJECT (importer->listener));
 		GNOME_Evolution_Importer_processItem (importer->importer,
@@ -370,17 +355,26 @@ importer_cb (EvolutionImporterListener *listener,
 static gboolean
 pine_import_file (PineImporter *importer,
 		  const char *path,
-		  const char *folderpath)
+		  const char *folderpath,
+		  gboolean folder)
 {
 	CORBA_boolean result;
 	CORBA_Environment ev;
 	CORBA_Object objref;
+	char *str;
 
 	CORBA_exception_init (&ev);
+
+	str = g_strdup_printf (_("Importing %s as %s"), path, folderpath);
+	gtk_label_set_text (GTK_LABEL (importer->label), str);
+	g_free (str);
+	while (gtk_events_pending ()) {
+		gtk_main_iteration ();
+	}
 	result = GNOME_Evolution_Importer_loadFile (importer->importer, path,
 						    folderpath, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION || result == FALSE) {
-		g_warning ("Exception here: %s", CORBA_exception_id (&ev));
+		g_warning ("Exception here: %s\n%s, %s", CORBA_exception_id (&ev), path, folderpath);
 		CORBA_Object_release (importer->importer, &ev);
 		CORBA_exception_free (&ev);
 		return FALSE;
@@ -408,16 +402,6 @@ pine_can_import (EvolutionIntelligentImporter *ii,
 	gboolean mail;
 	gboolean md_exists, addr_exists;
 
-	/* Already imported */
-
-	mail = bonobo_config_get_boolean_with_default (importer->db,
-                "/Importer/Pine/mail-imported", FALSE, NULL);
-
-	if (mail)
-		return FALSE;
-
-	importer->do_mail = !mail;
-
 	maildir = gnome_util_prepend_user_home ("mail");
 	md_exists = g_file_exists (maildir);
 	importer->do_mail = md_exists;
@@ -446,7 +430,7 @@ import_next (PineImporter *importer)
 		data = importer->dir_list->data;
 
 		folder = g_concat_dir_and_file (data->parent, data->foldername);
-		pine_import_file (importer, data->path, folder);
+		pine_import_file (importer, data->path, folder, data->folder);
 		g_free (folder);
 		g_free (data->parent);
 		g_free (data->path);
@@ -454,7 +438,7 @@ import_next (PineImporter *importer)
 		g_free (data);
 		importer->dir_list = importer->dir_list->next;
 	}
-
+	
 }
 
 /* Pine uses sent-mail and saved-mail whereas Evolution uses Sent and Drafts */
@@ -478,6 +462,7 @@ scan_dir (PineImporter *importer,
 	DIR *maildir;
 	struct stat buf;
 	struct dirent *current;
+	char *str;
 	
 	maildir = opendir (dirname);
 	if (maildir == NULL) {
@@ -486,6 +471,14 @@ scan_dir (PineImporter *importer,
 		return;
 	}
 	
+	str = g_strdup_printf (_("Scanning %s"), dirname);
+	gtk_label_set_text (GTK_LABEL (importer->label), str);
+	g_free (str);
+
+	while (gtk_events_pending ()) {
+		gtk_main_iteration ();
+	}
+
 	current = readdir (maildir);
 	while (current) {
 		PineFolder *pf;
@@ -520,15 +513,17 @@ scan_dir (PineImporter *importer,
 			pf->path = g_strdup (fullname);
 			pf->parent = g_strdup (orig_parent);
 			pf->foldername = g_strdup (foldername);
+			pf->folder = FALSE;
 			importer->dir_list = g_list_append (importer->dir_list, pf);
 		} else if (S_ISDIR (buf.st_mode)) {
 			char *subdir;
 
 			pf = g_new (PineFolder, 1);
-			pf->path = NULL;
+			pf->path = g_strdup (fullname);
 			pf->parent = g_strdup (orig_parent);
 			pf->foldername = g_strdup (foldername);
-			importer->dir_list = g_list_append (importer->dir_list, pf);
+			pf->folder = TRUE;
+  			importer->dir_list = g_list_append (importer->dir_list, pf);
 
 			subdir = g_concat_dir_and_file (orig_parent, foldername);
 			scan_dir (importer, fullname, subdir);
@@ -552,6 +547,13 @@ pine_create_structure (EvolutionIntelligentImporter *ii,
 	bonobo_object_ref (BONOBO_OBJECT (ii));
 	pine_store_settings (importer);
 
+	/* Create a dialog */
+	importer->dialog = create_importer_gui (importer);
+	gtk_widget_show_all (importer->dialog);
+	while (gtk_events_pending ()) {
+		gtk_main_iteration ();
+	}
+
 	if (importer->do_address == TRUE) {
 		bonobo_config_set_boolean (importer->db, 
                 "/Importer/Pine/address-imported", TRUE, NULL);
@@ -564,6 +566,12 @@ pine_create_structure (EvolutionIntelligentImporter *ii,
                 "/Importer/Pine/mail-imported", TRUE, NULL);
 
 		maildir = gnome_util_prepend_user_home ("mail");
+		gtk_label_set_text (GTK_LABEL (importer->label),
+				    _("Scanning directory"));
+		while (gtk_events_pending ()) {
+			gtk_main_iteration ();
+		}
+
 		scan_dir (importer, maildir, "/");
 		g_free (maildir);
 
@@ -621,13 +629,10 @@ checkbox_toggle_cb (GtkToggleButton *tb,
 static BonoboControl *
 create_checkboxes_control (PineImporter *importer)
 {
-	GtkWidget *container, *vbox;
+	GtkWidget *hbox;
 	BonoboControl *control;
 
-	container = gtk_frame_new (_("Import"));
-	vbox = gtk_vbox_new (FALSE, 2);
-	gtk_container_set_border_width (GTK_CONTAINER (container), 2);
-	gtk_container_add (GTK_CONTAINER (container), vbox);
+	hbox = gtk_hbox_new (FALSE, 2);
 
 	importer->mail = gtk_check_button_new_with_label (_("Mail"));
 	gtk_signal_connect (GTK_OBJECT (importer->mail), "toggled",
@@ -639,16 +644,11 @@ create_checkboxes_control (PineImporter *importer)
 			    GTK_SIGNAL_FUNC (checkbox_toggle_cb),
 			    &importer->do_address);
 
-	gtk_box_pack_start (GTK_BOX (vbox), importer->mail, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), importer->address, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), importer->mail, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), importer->address, FALSE, FALSE, 0);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (importer->mail), 
-				      importer->do_mail);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (importer->address),
-				      importer->do_address);
-
-	gtk_widget_show_all (container);
-	control = bonobo_control_new (container);
+	gtk_widget_show_all (hbox);
+	control = bonobo_control_new (hbox);
 	return control;
 }
 
@@ -688,7 +688,7 @@ factory_fn (BonoboGenericFactory *_factory,
 
 	importer = evolution_intelligent_importer_new (pine_can_import,
 						       pine_create_structure,
-						       _("Pine mail"),
+						       _("Pine"),
 						       _(message), pine);
 	gtk_signal_connect (GTK_OBJECT (importer), "destroy",
 			    GTK_SIGNAL_FUNC (pine_destroy_cb), pine);
