@@ -4,9 +4,8 @@
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,6 +57,52 @@ struct _EComponentRegistryPrivate {
 	GHashTable *component_id_to_component;
 };
 
+
+/* Utility functions.  */
+
+static int
+sleep_with_g_main_loop_timeout_callback (void *data)
+{
+	GMainLoop *loop;
+
+	loop = (GMainLoop *) data;
+	g_main_quit (loop);
+
+	return FALSE;
+}
+
+/* This function is like `sleep()', but it uses the GMainLoop so CORBA
+   invocations can get through.  */
+static void
+sleep_with_g_main_loop (int num_seconds)
+{
+	GMainLoop *loop;
+
+	loop = g_main_new (TRUE);
+	g_timeout_add (1000 * num_seconds, sleep_with_g_main_loop_timeout_callback, loop);
+	g_main_run (loop);
+	g_main_destroy (loop);
+}
+
+static void
+wait_for_corba_object_to_die (Bonobo_Unknown corba_objref,
+			      const char *id)
+{
+	gboolean alive;
+	int count;
+
+	count = 1;
+	while (1) {
+		alive = bonobo_unknown_ping (corba_objref);
+		if (! alive)
+			break;
+
+		g_print ("Waiting for component to die -- %s (%d)\n", id, count);
+		sleep_with_g_main_loop (1);
+		count ++;
+	}
+}
+
 
 /* Component information handling.  */
 
@@ -77,26 +122,36 @@ component_new (const char *id,
 	return new;
 }
 
-static void
+static gboolean
 component_free (Component *component)
 {
 	GNOME_Evolution_ShellComponent corba_shell_component;
 	CORBA_Environment ev;
+	gboolean retval;
 
 	CORBA_exception_init (&ev);
-	corba_shell_component = bonobo_object_corba_objref (BONOBO_OBJECT (component->client));
-	GNOME_Evolution_ShellComponent_unsetOwner (corba_shell_component, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION)
-		g_warning ("Cannot unregister component -- %s", component->id);
-	CORBA_exception_free (&ev);
 
-	g_free (component->id);
+	corba_shell_component = bonobo_object_corba_objref (BONOBO_OBJECT (component->client));
+	corba_shell_component = CORBA_Object_duplicate (corba_shell_component, &ev);
+
+	GNOME_Evolution_ShellComponent_unsetOwner (corba_shell_component, &ev);
+	if (ev._major == CORBA_NO_EXCEPTION)
+		retval = TRUE;
+	else
+		retval = FALSE;
+	CORBA_exception_free (&ev);
 
 	bonobo_object_unref (BONOBO_OBJECT (component->client));
 
+	wait_for_corba_object_to_die ((Bonobo_Unknown) corba_shell_component, component->id);
+	CORBA_Object_release (corba_shell_component, &ev);
+
 	e_free_string_list (component->folder_type_names);
+	g_free (component->id);
 
 	g_free (component);
+
+	return retval;
 }
 
 static gboolean
@@ -110,7 +165,8 @@ register_type (EComponentRegistry *component_registry,
 	       const char **exported_dnd_types,
 	       int num_accepted_dnd_types,
 	       const char **accepted_dnd_types,
-	       Component *handler)
+	       Component *handler,
+	       gboolean override_duplicate)
 {
 	EComponentRegistryPrivate *priv;
 	EFolderTypeRegistry *folder_type_registry;
@@ -120,6 +176,10 @@ register_type (EComponentRegistry *component_registry,
 	folder_type_registry = e_shell_get_folder_type_registry (priv->shell);
 	g_assert (folder_type_registry != NULL);
 
+	if (override_duplicate
+	    && e_folder_type_register_type_registered (folder_type_registry, name))
+		e_folder_type_register_unregister_type (folder_type_registry, name);
+
 	if (! e_folder_type_registry_register_type (folder_type_registry,
 						    name, icon_name, 
 						    display_name, description,
@@ -139,7 +199,8 @@ register_type (EComponentRegistry *component_registry,
 
 static gboolean
 register_component (EComponentRegistry *component_registry,
-		    const char *id)
+		    const char *id,
+		    gboolean override_duplicate)
 {
 	EComponentRegistryPrivate *priv;
 	GNOME_Evolution_ShellComponent component_corba_interface;
@@ -153,7 +214,7 @@ register_component (EComponentRegistry *component_registry,
 
 	priv = component_registry->priv;
 
-	if (g_hash_table_lookup (priv->component_id_to_component, id) != NULL) {
+	if (! override_duplicate && g_hash_table_lookup (priv->component_id_to_component, id) != NULL) {
 		g_warning ("Trying to register component twice -- %s", id);
 		return FALSE;
 	}
@@ -198,7 +259,8 @@ register_component (EComponentRegistry *component_registry,
 				     (const char **) type->exportedDndTypes._buffer,
 				     type->acceptedDndTypes._length,
 				     (const char **) type->acceptedDndTypes._buffer,
-				     component)) {
+				     component,
+				     override_duplicate)) {
 			g_warning ("Cannot register type `%s' for component %s",
 				   type->name, component->id);
 		}
@@ -218,8 +280,7 @@ register_component (EComponentRegistry *component_registry,
 			const CORBA_char *schema;
 
 			schema = supported_schemas->_buffer[i];
-			if (! e_uri_schema_registry_set_handler_for_schema (uri_schema_registry, schema, component->client))
-				g_warning ("Cannot register schema `%s' for component %s", schema, component->id);
+			e_uri_schema_registry_set_handler_for_schema (uri_schema_registry, schema, component->client);
 		}
 
 		CORBA_free (supported_schemas);
@@ -324,7 +385,7 @@ e_component_registry_register_component (EComponentRegistry *component_registry,
 	g_return_val_if_fail (E_IS_COMPONENT_REGISTRY (component_registry), FALSE);
 	g_return_val_if_fail (id != NULL, FALSE);
 
-	return register_component (component_registry, id);
+	return register_component (component_registry, id, FALSE);
 }
 
 
@@ -398,6 +459,56 @@ e_component_registry_get_component_by_id  (EComponentRegistry *component_registr
 	return component->client;
 }
 
+
+EvolutionShellComponentClient *
+e_component_registry_restart_component  (EComponentRegistry *component_registry,
+					 const char *id)
+{
+	EComponentRegistryPrivate *priv;
+	Component *component;
+	CORBA_Environment ev;
+	CORBA_Object corba_objref;
+
+	g_return_val_if_fail (component_registry != NULL, NULL);
+	g_return_val_if_fail (E_IS_COMPONENT_REGISTRY (component_registry), NULL);
+	g_return_val_if_fail (id != NULL, NULL);
+
+	priv = component_registry->priv;
+
+	component = g_hash_table_lookup (priv->component_id_to_component, id);
+	if (component == NULL)
+		return NULL;
+
+	CORBA_exception_init (&ev);
+
+	g_hash_table_remove (priv->component_id_to_component, id);
+
+	corba_objref = CORBA_Object_duplicate (bonobo_object_corba_objref (BONOBO_OBJECT (component->client)), &ev);
+
+	component_free (component);
+
+	wait_for_corba_object_to_die (corba_objref, id);
+
+	CORBA_exception_free (&ev);
+
+#if 1
+	if (! register_component (component_registry, id, TRUE))
+		return NULL;
+
+	return e_component_registry_get_component_by_id (component_registry, id);
+#else
+	client = evolution_shell_component_client_new (id);
+	if (client == NULL)
+		return NULL;
+
+	component = component_new (id, client);
+	g_hash_table_insert (priv->component_id_to_component, component->id, component);
+	bonobo_object_unref (BONOBO_OBJECT (client));
+#endif
+
+	return component->client;
+}
+
 
 E_MAKE_TYPE (e_component_registry, "EComponentRegistry", EComponentRegistry,
 	     class_init, init, PARENT_TYPE)
