@@ -4,9 +4,8 @@
  * Copyright (C) 2001  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,6 +26,8 @@
 
 #include "e-shell-user-creatable-items-handler.h"
 
+#include "e-util/e-corba-utils.h"
+
 #include <gal/util/e-util.h>
 
 #include <bonobo/bonobo-ui-util.h>
@@ -73,7 +74,7 @@ component_new_from_client (EvolutionShellComponentClient *client)
 	CORBA_exception_init (&ev);
 
 	objref = bonobo_object_corba_objref (BONOBO_OBJECT (client));
-	new->type_list = GNOME_Evolution_ShellComponent__get_user_creatable_item_types (objref, &ev);
+	new->type_list = GNOME_Evolution_ShellComponent__get_userCreatableItemTypes (objref, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION)
 		new->type_list = NULL;
